//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Comparison distribution
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
public class DistObject extends DataObject{

    // Supported distribution types
    static int UNIFORM      = 0;
    static int GAUSSIAN     = 1;
    static int BETA         = 2;
    static int LAST_DISTRIB = 3;
    int _type = GAUSSIAN;
    int _numBins;
    int _numDistribs;
    int _numSamples = 10000;
    double _minInt = -1.0; 
    double _maxInt =  1.0;
    double _stdDev = 1.0;

    // * * * Constructor * * * //
    DistObject(int numBins) { _numBins = numBins; }

    // * * * Return the number of distributions * * * //
    int getNumPts() { if(_histogram == null) return -1; else return _histogram.length; }

    // * * * Return the min/max interval value * * * //
    double getMinInterval(){ return _minInt; }
    double getMaxInterval(){ return _maxInt; }

    // * * * Set the min/mac interval value * * * //
    void setMinInterval(double min){ _minInt = min; }
    void setMaxInterval(double max){ _maxInt = max; }

    // * * * Return the std dev value * * * //
    double getStdDev(){ return _stdDev; }

    // * * * Set the std dev * * * //
    void setStdDev(double stdDev){ _stdDev = stdDev; }

    // * * * Change the number of bins * * * //
    void changeNumBins(int numBins){ _numBins = numBins; } 

    // * * * Change the type * * * //
    void changeType(int type, DataObject data) { _type = type; generateDistribution(data); }
    
    // * * * Get the type * * * //
    int getType() { return _type; }

    // * * * Get the type label * * * //
    String getLabel() {
	if(_type == UNIFORM)        return "Uniform";
	else if(_type == GAUSSIAN)  return "Gaussian";
	else if(_type == BETA)      return "Beta";
	else                        return "Unknown";
    }

    // * * * Generate the distribution * * * //
    void generateDistribution(DataObject data){

	_numDistribs = data.getNumPts();
	_minDatas = data.getMinDatas();
	_maxDatas = data.getMaxDatas();
	_numBins = data.getNumBins();
	_histogram    = new double[_numDistribs][_numBins];
	_minDensities = new double[_numDistribs];
	_maxDensities = new double[_numDistribs];
	_means        = new double[_numDistribs];

	if(_type == UNIFORM){ generateUniform(); 	}				       
	else if(_type == GAUSSIAN){ generateGaussian(data); }
        else if(_type == BETA){ generateBeta(data); }
    }

    // * * * Create uniform distribution * * * //
    void generateUniform() {
	for(int i = 0; i < _numDistribs; i++) {
	    double interval = _maxDatas[i] - _minDatas[i];
	    if(interval == 0.0) interval = 1.0;
	    for(int j = 0; j < _numBins; j++) 
		_histogram[i][j] = 1.0/(_numBins);
	    _minDensities[i] = _maxDensities[i] = _histogram[i][0];
	    _means[i] = _minDatas[i] + (_maxDatas[i]-_minDatas[i])*.5;
	}
    }
    
    // * * * Generate a Gaussian distribution * * * //
    void generateGaussian(DataObject data) {
	for(int i = 0; i < _numDistribs; i++) {
	    double mean = Etc.affine(_minDatas[i], data.getMean(i), _maxDatas[i], 0, _numBins-1);
	    double interval = _maxDatas[i] - _minDatas[i];
	    double sum = 0.0;
	    _stdDev = data.getStdDev(i) / (data.getMaxData(i)-data.getMinData(i))*(_numBins-1);
	    if(_minDatas[i] == _maxDatas[i]){_stdDev = -1;}
	    for(int j = 0; j < _numBins; j++) {
                if(_stdDev == -1){
		    _histogram[i][j] = 1;
		}
		else{
		  _histogram[i][j] = 1/Math.sqrt(2*Math.PI*_stdDev) * Math.exp( -((j-mean)*(j-mean))/(2*_stdDev*_stdDev) ) ;
		}
                sum += _histogram[i][j];
	    }
	    _means[i] = data.getMean(i);
	    _minDensities[i] = _maxDensities[i] = _histogram[i][0]/sum;
	    for(int j=0; j< _numBins; j++){
		_histogram[i][j] /= sum;
		_maxDensities[i] = _maxDensities[i] > _histogram[i][j] ? _maxDensities[i] : _histogram[i][j];
		_minDensities[i] = _minDensities[i] < _histogram[i][j] ? _minDensities[i] : _histogram[i][j];
	    }
	}
    }  
    // * * * Generate a Gaussian distribution * * * //
    void generateBeta(DataObject data) {
	for(int i = 0; i < _numDistribs; i++) {
	    double mean = Etc.affine(_minDatas[i], data.getMean(i), _maxDatas[i], 0, 1);
	    double interval = _maxDatas[i] - _minDatas[i];
	    double sum = 0.0;
	    _stdDev = data.getStdDev(i) / (data.getMaxData(i)-data.getMinData(i));
	    double v=_stdDev*_stdDev;

            double tmp = mean*(1-mean)/v -1;
            double alpha = mean * tmp;
            double beta = (1-mean)*tmp;
            if(_minDatas[i] == _maxDatas[i]){_stdDev = -1;}
	    for(int j = 0; j < _numBins; j++) {
                if(_minDatas[i]  == _maxDatas[i]){
		    _histogram[i][j] = 1;
		}
		else{
		    double x= (j+0.5)/_numBins;
		    _histogram[i][j] = Math.pow(x, alpha-1 ) * Math.pow(1-x, beta-1) ;
		}
                sum += _histogram[i][j];
	    }
	    _means[i] = data.getMean(i);
	    _minDensities[i] = _maxDensities[i] = _histogram[i][0]/sum;
	    for(int j=0; j< _numBins; j++){
		_histogram[i][j] /= sum;
		_maxDensities[i] = _maxDensities[i] > _histogram[i][j] ? _maxDensities[i] : _histogram[i][j];
		_minDensities[i] = _minDensities[i] < _histogram[i][j] ? _minDensities[i] : _histogram[i][j];
	    }
	}
    }
}