//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Class to compute the distance between 2 PDFs
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
final int L1 = 0;
final int HELLINGER = 1;
final int L2 = 2;
final int LAST_MEASURE = 3;
String TYPE_NAMES[] = {"L1 Norm", "Hellinger", "L2 Norm"};

class PdfDistance {
    double _distances[];   
    double _minDist, _maxDist;

    double _intervals[];
    double _minInt, _maxInt;

    double _minStd, _maxStd;

    int _type = L1;
    DataObject _obj1, _obj2;

    int _numBins;
    int _numPts;

    // * * * Constructor * * * //
    PdfDistance(DataObject obj1, DataObject obj2, int type) {
	_type = type;
	_obj1 = obj1; _obj2 = obj2;
	_numBins = obj1.getNumBins(); _numPts = obj1.getNumPts();
	calculateDistances();
	calculateIntervals();
    }

    // * * * Constructor (defulat type) * * * //
    PdfDistance(DataObject obj1, DataObject obj2) { this(obj1, obj2, L1); }

    // * * * Get the distances * * * //
    double [] getDistances()  { return _distances; } 
    double getDistance(int i) { return _distances[i]; }

    // * * * Get the min/max distances * * *//
    double getMaxDist() { return _maxDist; }
    double getMinDist() { return _minDist; }

    // * * * Get the intervals * * * //
    double [] getIntervals()  { return _intervals; }
    double getInterval(int i) { return _intervals[i]; }

    // * * * Get the min/max intervals * * * //
    double getMinInterval() { return _minInt; }
    double getMaxInterval() { return _maxInt; }

    // * * * Get the min/max standard deviations * * * //
    double getMinStd() { return _minStd; }
    double getMaxStd() { return _maxStd; }

    // * * * Get the type as a string * * * //
    String getTypeName() { return TYPE_NAMES[_type]; }
   
    // * * * Get the type as an integer * * * //
    int getType(){ return _type; }

    // * * * Toggle the type * * * //
    void toggleMeasure() { _type ++; if(_type >= LAST_MEASURE) _type = 0; calculateDistances(); }

    // * * * Calculate the intervals * * * //
    void calculateIntervals(){
	_intervals = new double[_numPts];
	for(int i = 0; i < _numPts; i++)  _intervals[i] = _obj1.getMaxData(i) - _obj1.getMinData(i);
      	_minInt = _maxInt = _intervals[0];
	for(int i = 0; i < _numPts; i++){
	    _minInt = _minInt < _intervals[i] ? _minInt : _intervals[i];
	    _maxInt = _maxInt > _intervals[i] ? _maxInt : _intervals[i];
	}
    }

    // * * * Calculate the distance between the pdfs * * * //
    void calculateDistances(){
	// Go through each pdf in the data
	_distances = new double[_numPts];
	boolean start = true;
	for(int i = 0; i < _numPts; i++) {
	    _distances[i] = calculateDistanceDef(_obj1.getHistogram(i), 
						 _obj2.getHistogram(i));
	    if(start) { 
		_minDist = _maxDist = _distances[i]; 
		_minStd = _maxStd = _obj1.getStdDev(0); 
		start = false; 
	    }
	    _minDist = _minDist < _distances[i] ? _minDist : _distances[i];
	    _maxDist = _maxDist > _distances[i] ? _maxDist : _distances[i];   
	    _minStd = _minStd < _obj1.getStdDev(i) ? _minStd : _obj1.getStdDev(i);
	    _maxStd = _maxStd > _obj1.getStdDev(i) ? _maxStd : _obj1.getStdDev(i);
	}
    }
    
    // * * * Do the actual calculation * * * //
    double calculateDistanceDef(double pdf1[], double pdf2[]) {
	double distance = -1;
	switch(_type){
	case L1:
	    distance =  L1Norm(pdf1, pdf2);
	    break;
	case L2:
	    distance = L2Norm(pdf1, pdf2);
	    break;
	case HELLINGER:
	    distance =  Hellinger(pdf1, pdf2);
	    break;
	default:
	    println("PdfDistance::calculateDistance: ERROR! Distance type not recognized!");
	    break;
	}
	return distance;
    }

    // * * * L1 norm * * * //
    double L1Norm(double pdf1[], double pdf2[]) {
	double sum = 0.0;
	for(int i = 0; i < _numBins; i++) sum += abs((float)pdf1[i] - (float)pdf2[i]);
	return sum/(double)_numBins;
    }

    // * * * L2 norm * * * //
    double L2Norm(double pdf1[], double pdf2[]) {
	double sum = 0;
	for(int i = 0; i < _numBins; i++) sum += (pdf1[i] - pdf2[i])*(pdf1[i] - pdf2[i]);
	return sqrt((float)sum/_numBins);
    }

    // * * * Hellinger * * * //
    double Hellinger(double pdf1[], double pdf2[]) {
	double sum = 0;
	for(int i = 0; i < _numBins; i++){
	    double sqrtDiff = Math.sqrt(pdf1[i]) - Math.sqrt(pdf2[i]);
	    sum += (sqrtDiff*sqrtDiff);
	}	return sqrt((float)sum/(2*_numBins));
    }

   
}